/* 	appVOC - Application of voice call service
 *
 *	Authors: JIANG Dan <dan.jiang@orange-ftgroup.com> 
 *	
 *	This software and associated documentation files (the "Software") 
 *	are copyright (C) 2005 LiPS Linux Phone Standards Forum [FranceTelecom] 
 *	All Rights Reserved. 
 *
 *	A copyright license is hereby granted for redistribution and use of 
 *	the Software in source and binary forms, with or without modification, 
 *	provided that the following conditions are met: 
 *	- Redistributions of source code must retain the above copyright notice, 
 *	this copyright license and the following disclaimer. 
 *- Redistributions in binary form must reproduce the above copyright 
 *	notice, this copyright license and the following disclaimer in the 
 *	documentation and/or other materials provided with the distribution. 
 *	- Neither the name of LiPS  nor the names of its Members may be used 
 *	to endorse or promote products derived from the Software without 
 *	specific prior written permission. 
 *
 *	A patent license for any Necessary Claims owned by Members of LiPS Forum 
 *	to make, have made, use, import, offer to sell, lease and sell or otherwise 
 *	distribute any implementation compliant with the any specification adopted 
 *	by the LiPS Forumcan be obtained from the respective Members on reasonable 
 *	and non-discriminatory terms and conditions and under reciprocity, as 
 *	regulated in more detail in the Internal Policy of the LiPS Forum. 
 *
 *	THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDER, ITS MEMBERS AND CONTRIBUTORS 
 *	"AS IS", AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, 
 *	THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE 
 *	AND NON-INFRINGEMENT ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER, 
 *	ITS MEMBERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *	SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, 
 *	PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; 
 *	OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 *	WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 *	ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
 *	POSSIBILITY OF SUCH DAMAGE. 
 */


#include "linestate.h"
#include "inputwindow.h"
#include "connectedwindow.h"
#include "msgdialog.h"
#include <voc.h>

static struct
{
  GtkWidget *window;
  gchar *call_name;
  gchar *call_num;
  guint cid;
  GtkWidget *label_num;
  GtkWidget *identity_image;
  GdkPixmap *pixmap_calling;
  GdkPixbuf *pixbuf_calling;
  gint timerid;
  gboolean have_softkeys;
  gint num_softkeys;
  GtkActionGroup *action_group;
  GtkUIManager *ui_manager;
  GtkWidget *menu_left;
  GtkWidget *menu_right;
} self;
gint image_change_count = 0;
static void
softkey_activated (GtkWidget *window, gint nr)
{
  if (nr == 1)
  {

    if (!iswifi_flag)
    {
      voc_call_evt_unregister (0, self.cid, 4);
      //voc_gsm_hungup_outgoing (0, self.cid);
      voc_hungup_outgoing (0, self.cid);

    }
    else
    {
      voc_call_evt_unregister (1, self.cid, 4);
      haswifi_flag=FALSE;
//      voc_wifi_hungup (1, 60001);
      voc_hungup (1, 60001);	  
    }
    gtk_window_iconify (GTK_WINDOW (window));
    g_free (self.call_name);
    if (self.timerid)
      g_source_remove (self.timerid);
  }
}
static gboolean
window_key_press_event (GtkWidget *widget, GdkEventKey *k, gpointer data)
{
  switch (k->keyval)
  {
    case KEY_END:
      softkey_activated (self.window, 1);
      return TRUE;
    default:
      break;
  }
  return FALSE;
}


void
handler_exception ()
{
  g_print ("in %s\n", __FUNCTION__);
  voc_call_evt_unregister (1, 60001, 4);
  g_print ("in %s, after unregister\n", __FUNCTION__);
  gtk_window_iconify (GTK_WINDOW (self.window));
  g_free (self.call_name);
  if (self.timerid)
    g_source_remove (self.timerid);
  create_msgdialog ("Wrong number!", MSG_NORMAL_TIMEOUT);
}


void
on_connected (guint cid)
{
  addnewcall (cid, self.call_name, self.call_num, ACTIVE, self.identity_image);
  g_print ("in on_connected\n");
  g_print ("number=%s,name=%s\n", self.call_num, self.call_name);

  show_connectedwindow ();
  if (self.timerid)
    g_source_remove (self.timerid);
}

static gboolean
timer_handle ()
{
  image_change_count++;
  if (image_change_count == 4)
    image_change_count = 0;
  gint width, height;

  width = gdk_pixbuf_get_width (self.pixbuf_calling);
  height = gdk_pixbuf_get_height (self.pixbuf_calling);
  gdk_draw_pixbuf (self.pixmap_calling, NULL, self.pixbuf_calling,
                   width - 66, height - 30, width - 66, height - 30, 66, 30,
                   GDK_RGB_DITHER_NORMAL, 1, 1);
  gdk_draw_rectangle (self.pixmap_calling, self.window->style->white_gc,
                      TRUE, width - (3 - image_change_count) *22,
                      height - 30, (3 - image_change_count) *22, 30);

  gtk_widget_queue_draw (self.window);
  return TRUE;
}

static gboolean
setup_softkeys (void)
{
  self.num_softkeys =
    gpe_skwindow_get_softkey_count (GPE_SKWINDOW (self.window));
  if (self.num_softkeys == 2)
  {
    gpe_skwindow_set_softkey_label (GPE_SKWINDOW (self.window), SK_LEFT, _(""));
    gpe_skwindow_set_softkey_label (GPE_SKWINDOW (self.window), SK_RIGHT,
                                    _("Stop"));
    return TRUE;
  }
  return FALSE;
}

GtkWidget *
get_callingwindow ()
{
  return self.window;
}

guint
get_current_cid ()
{
  return self.cid;
}
static void
update_callingwindow (gchar *num, gchar *name, gchar *photo_path)
{
  image_change_count = 0;
  if (self.timerid)
    g_source_remove (self.timerid);
  self.timerid = g_timeout_add (500, (GSourceFunc) timer_handle, NULL);

  self.call_num = num;
  self.call_name = name;
  gchar *temp_string;

  if (strcmp (name, "Unknown"))
    temp_string = name;
  else
    temp_string = num;
  GString *temp = g_string_new (temp_string);

  if (temp->len > 11)
  {
    temp = g_string_truncate (temp, 7);
    g_string_append (temp, "...");
  }
  gtk_label_set_text (GTK_LABEL (self.label_num), temp->str);
  g_string_free (temp, TRUE);
  GdkPixbuf *pbuf = NULL;

  if (photo_path)
  {
    pbuf =
      gdk_pixbuf_new_from_file_at_size (photo_path, PHOTO_SIZE_W,
                                        PHOTO_SIZE_W, NULL);
    g_free (photo_path);
  }

  if (!pbuf)
    pbuf = default_photo;
  gtk_image_set_from_pixbuf (GTK_IMAGE (self.identity_image), pbuf);

}

void
show_callingwindow (gchar *num, gchar *name, gchar *photo_path)
{
  guint cid;

  if (!num)
    return;
  if (!name || !strcmp (name, ""))
    name = g_strdup ("Unknown");

  if (ACTIVECALLINDEX != -1)
  {
//    if (!voc_gsm_onhold
    if (!voc_onhold
        (0, g_array_index (call_array, call_desc, ACTIVECALLINDEX).cid))
    {
      voc_call_evt_register (0, g_array_index (call_array, call_desc, ACTIVECALLINDEX).cid, VOC_EVT_CALL_STATUS, NULL, voc_cb_onhold, NULL);    //cb1 is used to change the UI, etc.

    }
  }
  char buf[50];
  char *rnum = buf;
  int appsid;

  if (num[0] == '9')
  {

    g_print ("!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!rnum=%s\n", num);
    sprintf (buf, "sip:%s@10.193.251.160", ++num);
    rnum = buf;
    g_print ("!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!rnum=%s\n", rnum);
    appsid = 1;
    iswifi_flag = TRUE;
    haswifi_flag = TRUE;

  }
  else if (num[0] == '8')
  {
#ifdef WANG
    char buf1[] = "sip:dan@192.168.1.5:5060";

    rnum = buf1;
#endif

#ifdef YANG
    char buf2[] = "sip:root@192.168.1.2";

    rnum = buf2;
#endif
    g_print ("!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!rnum=%s\n", rnum);
    appsid = 1;
    iswifi_flag = TRUE;
    haswifi_flag = TRUE;

  }


  else
  {
    rnum = num;
    appsid = 0;
    iswifi_flag = FALSE;
  }

  g_print ("in function%s,appsid=%d\n", __FUNCTION__, appsid);
//JDD
  if (voc_dial (appsid, 1, &cid, g_strdup (rnum)))      //jd
  {
    printf ("DIAL error!\n");
    create_msgdialog ("Can't create a new call!", MSG_NORMAL_TIMEOUT);
    return;
  }
  else
  {
    printf ("in %s, appsid=%d,cid=%d\n", __FUNCTION__, appsid, cid);
    g_print ("voc_cb_dial:0x%08x", (guint32) voc_cb_dial);
    voc_call_evt_register (appsid, cid, VOC_EVT_CALL_STATUS, NULL, voc_cb_dial, g_strdup (rnum));       //cb1 is used to change the UI, etc.

    self.cid = cid;
  }
  g_print ("before call history\n");
  voc_call_history_add_history (rnum, OUTGOING, 0);
  g_print ("after call history\n");

  if (self.window)
  {
    update_callingwindow (num, name, photo_path);
    gtk_widget_show_all (self.window);
    gtk_window_deiconify (GTK_WINDOW (self.window));

    gtk_window_present (GTK_WINDOW (self.window));
    return;
  }
  GtkWidget *identity_image = NULL;
  GtkWidget *mainwindow;
  GtkWidget *mainbox, *label1, *label2;
  GtkWidget *image;
  GtkWidget *hbox;
  PangoFontDescription *font_desc;

  if (!self.pixmap_calling || !self.pixbuf_calling)
  {
    gchar dir[100];

    strcpy (dir, IMAGE_DIR);
    strcat (dir, "/calling.png");
    load_image (dir, &self.pixbuf_calling, &self.pixmap_calling, NULL, NULL);
  }

  mainwindow = gpe_skwindow_new ();
  self.window = mainwindow;
  gtk_window_set_default_size (GTK_WINDOW (mainwindow), 300, 400);
  gtk_window_set_title (GTK_WINDOW (mainwindow), "Calling ");
  gtk_widget_modify_bg (mainwindow, GTK_STATE_NORMAL, &white);
  g_signal_connect (mainwindow, "destroy",
                    G_CALLBACK (gtk_widget_destroyed), &self.window);
  g_signal_connect (G_OBJECT (mainwindow), "key_press_event",
                    G_CALLBACK (window_key_press_event), NULL);
  g_signal_connect (G_OBJECT (mainwindow), "softkey_activated",
                    G_CALLBACK (softkey_activated), NULL);
  image = gtk_image_new_from_pixmap (self.pixmap_calling, NULL);
  label1 = gtk_label_new ("Calling...");
  label2 = gtk_label_new ("Calling...");
  identity_image = gtk_image_new ();
  self.label_num = label2;
  self.identity_image = identity_image;
  gtk_label_set_justify (GTK_LABEL (label1), GTK_JUSTIFY_CENTER);
  gtk_label_set_justify (GTK_LABEL (label2), GTK_JUSTIFY_CENTER);
  gtk_widget_set_size_request (image, 200, 120);
  gtk_misc_set_alignment (GTK_MISC (image), 0.5, 1);
  gtk_misc_set_alignment (GTK_MISC (label1), 0.5, 1);
  gtk_misc_set_alignment (GTK_MISC (label2), 0.8, 0.2);
  gtk_misc_set_alignment (GTK_MISC (identity_image), 0.3, 0);

  font_desc = pango_font_description_from_string ("Sans Bold 15");
  gtk_widget_modify_font (label1, font_desc);
  pango_font_description_free (font_desc);
  font_desc = pango_font_description_from_string ("Sans Bold 18");
  gtk_widget_modify_font (label2, font_desc);
  pango_font_description_free (font_desc);



  mainbox = gtk_vbox_new (FALSE, 0);
  hbox = gtk_hbox_new (FALSE, 0);
  gtk_box_pack_start (GTK_BOX (hbox), label2, TRUE, TRUE, 0);
  gtk_box_pack_start (GTK_BOX (hbox), identity_image, TRUE, TRUE, 0);

  gtk_box_pack_start (GTK_BOX (mainbox), image, FALSE, FALSE, 0);
  gtk_box_pack_start (GTK_BOX (mainbox), label1, FALSE, FALSE, 0);
  gtk_box_pack_start (GTK_BOX (mainbox), hbox, TRUE, TRUE, 0);

  gtk_container_add (GTK_CONTAINER (mainwindow), mainbox);
  self.have_softkeys = setup_softkeys ();
  update_callingwindow (num, name, photo_path);
  gtk_widget_show_all (mainwindow);

}

/* 
vi:ts=2:nowrap:ai:expandtab
*/

