/* 	appVOC - Application of voice call service
 *
 *	Authors: JIANG Dan <dan.jiang@orange-ftgroup.com> 
 *	
 *	This software and associated documentation files (the "Software") 
 *	are copyright (C) 2005 LiPS Linux Phone Standards Forum [FranceTelecom] 
 *	All Rights Reserved. 
 *
 *	A copyright license is hereby granted for redistribution and use of 
 *	the Software in source and binary forms, with or without modification, 
 *	provided that the following conditions are met: 
 *	- Redistributions of source code must retain the above copyright notice, 
 *	this copyright license and the following disclaimer. 
 *- Redistributions in binary form must reproduce the above copyright 
 *	notice, this copyright license and the following disclaimer in the 
 *	documentation and/or other materials provided with the distribution. 
 *	- Neither the name of LiPS  nor the names of its Members may be used 
 *	to endorse or promote products derived from the Software without 
 *	specific prior written permission. 
 *
 *	A patent license for any Necessary Claims owned by Members of LiPS Forum 
 *	to make, have made, use, import, offer to sell, lease and sell or otherwise 
 *	distribute any implementation compliant with the any specification adopted 
 *	by the LiPS Forumcan be obtained from the respective Members on reasonable 
 *	and non-discriminatory terms and conditions and under reciprocity, as 
 *	regulated in more detail in the Internal Policy of the LiPS Forum. 
 *
 *	THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDER, ITS MEMBERS AND CONTRIBUTORS 
 *	"AS IS", AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, 
 *	THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE 
 *	AND NON-INFRINGEMENT ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER, 
 *	ITS MEMBERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *	SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, 
 *	PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; 
 *	OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 *	WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 *	ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
 *	POSSIBILITY OF SUCH DAMAGE. 
 */

#include "linestate.h"
#include "callingwindow.h"
#include "icmc.h"
#include "interface.h"
#include "msgdialog.h"
#include <voc.h>
static struct
{
  GtkWidget *window;
  GtkTextBuffer *buffer;
  gboolean have_softkeys;
  gint num_softkeys;
  gchar *name;
  gchar *photo_path;
  gchar *prenum;
  GtkActionGroup *action_group;
  GtkUIManager *ui_manager;
  GtkWidget *menu_left;
  GtkWidget *menu_right;
} self;
static void call (void);
static void save (void);
static void find (void);


static const gchar *ui_description = " \
    <ui>\
    <popup  action=\"PopupLeft\">\
    <menuitem name=\"call\" action=\"Call\"/>\
    <menuitem name=\"Find\" action=\"Find\"/>\
    </popup>\
    </ui>\
    ";


static const GtkActionEntry action_entries[] = {
  {"Call", NULL, "_Call                      ", "", "Call",
   G_CALLBACK (call)},
  //{"Save", NULL, "_Save", "", "Save", G_CALLBACK(save)},
  {"Find", NULL, "_Find..", "", "Find", G_CALLBACK (find)},
  {"PopupLeft", NULL, "_Popup", "", "Popup", NULL},
  {"PopupRight", NULL, "_Popup", "", "Popup", NULL},
};
static void
call (void)
{
  GtkTextIter start, end;

  gtk_text_buffer_get_bounds (self.buffer, &start, &end);
  gchar *num = gtk_text_buffer_get_text (self.buffer, &start, &end, FALSE);

  if (strlen (num) == 0)
  {
    create_msgdialog ("please input the num!", MSG_NORMAL_TIMEOUT);
    return;
  }
  search_from_addressbook (num, &self.name, &self.photo_path, NULL);

  g_print ("\t\t\tAddress success!!!!!!!!!!!!!!!\n\n\n");

  show_callingwindow (num, self.name, self.photo_path);
}
static void
save (void)
{
}
static void
find (void)
{
  invoke_addressbook_ui_iac ();
}
static void
init_menu (void)
{
  GError *err = NULL;

  self.action_group = gtk_action_group_new ("AllActions");
  self.ui_manager = gtk_ui_manager_new ();

  gtk_action_group_add_actions (self.action_group, action_entries,
                                G_N_ELEMENTS (action_entries), NULL);
  gtk_ui_manager_insert_action_group (self.ui_manager, self.action_group, 0);
  if (!gtk_ui_manager_add_ui_from_string
      (self.ui_manager, ui_description, -1, &err))
  {
    g_printerr ("Failed to parse menu data: %s\n", err->message);
    g_error_free (err);
    return;
  }
  self.menu_left = gtk_ui_manager_get_widget (self.ui_manager, "/PopupLeft");
  self.menu_right = NULL;
#if 1
  g_signal_connect (self.menu_left, "key-press-event",
                    G_CALLBACK (popup_menu_key_press_handler), NULL);
#endif
  gpe_skwindow_set_softkey_menu (GPE_SKWINDOW (self.window), SK_LEFT,
                                 self.menu_left);
  gpe_skwindow_set_softkey_menu (GPE_SKWINDOW (self.window), SK_RIGHT,
                                 self.menu_right);
}
static void
softkey_activated (GtkWidget *window, gint nr)
{
  if (nr == 0)
  {
    skwindow_dbus_set_softkey_label (0, "OK");
    skwindow_dbus_set_softkey_label (1, "Cancel");
  }
  if (nr == 1)
  {
    printf ("right key pressed and hided\n");
    gtk_window_iconify (GTK_WINDOW (window));
  }
}
static gboolean
window_key_press_event (GtkWidget *widget, GdkEventKey *k, gpointer data)
{


  if (k->keyval <= GDK_9 && k->keyval >= GDK_0)
    return FALSE;
  GtkTextIter iter;
  switch (k->keyval)
  {                             //k->keyval ==KEY_STAR||k->keyval ==KEY_#
    case KEY_SEND:
      call ();
      return TRUE;
    case GDK_BackSpace:
      return FALSE;
    case 44:
      printf("* is inputed\n");
      gtk_text_buffer_get_end_iter    (self.buffer,&iter);
      gtk_text_buffer_insert(self.buffer,&iter,"*",-1);
      return TRUE;
    case 46:
      printf("# is inputed\n");
      gtk_text_buffer_get_end_iter    (self.buffer,&iter);
      gtk_text_buffer_insert(self.buffer,&iter,"#",-1);
      return TRUE;
    default:
      break;
  }
  return TRUE;
}

static gboolean
setup_softkeys (void)
{
  self.num_softkeys =
    gpe_skwindow_get_softkey_count (GPE_SKWINDOW (self.window));
  if (self.num_softkeys == 2)
  {
    gpe_skwindow_set_softkey_label (GPE_SKWINDOW (self.window), SK_LEFT,
                                    _("Option"));
    gpe_skwindow_set_softkey_label (GPE_SKWINDOW (self.window), SK_RIGHT,
                                    _("Back"));
    return TRUE;
  }
  return FALSE;
}

GtkWidget *
get_inputwindow ()
{
  return self.window;
}

GtkWidget *
show_inputwindow (gchar *num, gchar *name, gchar *photo_path)
{
  if (self.window)
  {
    if (!num)
      num = "";
    gtk_text_buffer_set_text (self.buffer, num, -1);
    self.name = name;
    self.photo_path = photo_path;
    self.have_softkeys = setup_softkeys ();
    gtk_widget_show_all (self.window);
    gtk_window_deiconify (self.window);
    gtk_window_present (GTK_WINDOW (self.window));
    return self.window;
  }
  GtkWidget *mainwindow;
  PangoFontDescription *font_desc;
  GtkWidget *mainbox, *textview, *separator, *obligatebox;

  mainwindow = gpe_skwindow_new ();

  self.window = mainwindow;
  self.name = name;
  self.photo_path = photo_path;

  gtk_window_set_default_size (GTK_WINDOW (mainwindow), 300, 400);
  g_signal_connect (G_OBJECT (mainwindow), "key_press_event",
                    G_CALLBACK (window_key_press_event), NULL);
  g_signal_connect (mainwindow, "destroy",
                    G_CALLBACK (gtk_widget_destroyed), &self.window);
  gtk_window_set_title (GTK_WINDOW (mainwindow), "Input num:");
  gtk_widget_realize (mainwindow);

  self.buffer = gtk_text_buffer_new (NULL);
  textview = gtk_text_view_new_with_buffer (self.buffer);
  font_desc = pango_font_description_from_string ("Serif 20");
  gtk_widget_modify_font (textview, font_desc);
  pango_font_description_free (font_desc);
  gtk_text_view_set_cursor_visible (GTK_TEXT_VIEW (textview), TRUE);
  gtk_text_view_set_wrap_mode (GTK_TEXT_VIEW (textview), GTK_WRAP_WORD_CHAR);
  gtk_text_view_set_justification (GTK_TEXT_VIEW (textview),
                                   GTK_JUSTIFY_CENTER);

  GTK_WIDGET_SET_FLAGS (textview, GTK_HAS_FOCUS);
  gtk_text_view_set_pixels_above_lines (GTK_TEXT_VIEW (textview), 10);
  gtk_text_view_set_pixels_below_lines (GTK_TEXT_VIEW (textview), 10);

  if (!GTK_WIDGET_REALIZED (mainwindow))
    gtk_widget_realize (mainwindow);
  gtk_widget_set_app_paintable (mainwindow, TRUE);
  gdk_window_set_back_pixmap (mainwindow->window, bg_pixmap, FALSE);
  gtk_widget_queue_draw (mainwindow);
  separator = gtk_hseparator_new ();
  mainbox = gtk_vbox_new (FALSE, 0);
  obligatebox = gtk_vbox_new (FALSE, 0);


  g_signal_connect (G_OBJECT (mainwindow), "softkey_activated",
                    G_CALLBACK (softkey_activated), NULL);
  gtk_box_pack_start (GTK_BOX (mainbox), textview, FALSE, FALSE, 0);
  gtk_box_pack_start (GTK_BOX (mainbox), separator, FALSE, FALSE, 0);
  gtk_box_pack_start (GTK_BOX (mainbox), obligatebox, TRUE, TRUE, 0);
  gtk_container_add (GTK_CONTAINER (mainwindow), mainbox);


  init_menu ();
  self.have_softkeys = setup_softkeys ();
  if (!num)
    num = "";
  gtk_text_buffer_set_text (self.buffer, num, -1);
  gtk_widget_show_all (mainwindow);
  return mainwindow;

}

GtkWidget *
show_inputwindow_hide (gchar *num, gchar *name, gchar *photo_path)
{
  if (self.window)
    return self.window;

  GtkWidget *mainwindow;
  PangoFontDescription *font_desc;
  GtkWidget *mainbox, *textview, *separator, *obligatebox;

  mainwindow = gpe_skwindow_new ();

  self.window = mainwindow;
  self.name = name;
  self.photo_path = photo_path;

  gtk_window_set_default_size (GTK_WINDOW (mainwindow), 300, 400);
  g_signal_connect (G_OBJECT (mainwindow), "key_press_event",
                    G_CALLBACK (window_key_press_event), NULL);
  g_signal_connect (mainwindow, "destroy",
                    G_CALLBACK (gtk_widget_destroyed), &self.window);
  gtk_window_set_title (GTK_WINDOW (mainwindow), "Input num:");
  gtk_widget_realize (mainwindow);

  self.buffer = gtk_text_buffer_new (NULL);
  textview = gtk_text_view_new_with_buffer (self.buffer);
  font_desc = pango_font_description_from_string ("Serif 20");
  gtk_widget_modify_font (textview, font_desc);
  pango_font_description_free (font_desc);
  gtk_text_view_set_cursor_visible (GTK_TEXT_VIEW (textview), TRUE);
  gtk_text_view_set_wrap_mode (GTK_TEXT_VIEW (textview), GTK_WRAP_WORD_CHAR);
  gtk_text_view_set_justification (GTK_TEXT_VIEW (textview),
                                   GTK_JUSTIFY_CENTER);

  GTK_WIDGET_SET_FLAGS (textview, GTK_HAS_FOCUS);
  gtk_text_view_set_pixels_above_lines (GTK_TEXT_VIEW (textview), 10);
  gtk_text_view_set_pixels_below_lines (GTK_TEXT_VIEW (textview), 10);

  if (!GTK_WIDGET_REALIZED (mainwindow))
    gtk_widget_realize (mainwindow);
  gtk_widget_set_app_paintable (mainwindow, TRUE);
  gdk_window_set_back_pixmap (mainwindow->window, bg_pixmap, FALSE);
  gtk_widget_queue_draw (mainwindow);
  separator = gtk_hseparator_new ();
  mainbox = gtk_vbox_new (FALSE, 0);
  obligatebox = gtk_vbox_new (FALSE, 0);


  g_signal_connect (G_OBJECT (mainwindow), "softkey_activated",
                    G_CALLBACK (softkey_activated), NULL);
  gtk_box_pack_start (GTK_BOX (mainbox), textview, FALSE, FALSE, 0);
  gtk_box_pack_start (GTK_BOX (mainbox), separator, FALSE, FALSE, 0);
  gtk_box_pack_start (GTK_BOX (mainbox), obligatebox, TRUE, TRUE, 0);
  gtk_container_add (GTK_CONTAINER (mainwindow), mainbox);


  init_menu ();
  //self.have_softkeys = setup_softkeys ();
  gtk_widget_realize (mainwindow);
  gtk_widget_realize (mainbox);
  gtk_widget_realize (textview);
  gtk_widget_realize (separator);
  gtk_widget_realize (obligatebox);
  return mainwindow;

}

/* 
vi:ts=2:nowrap:ai:expandtab
*/

