/* 	appVOC - Application of voice call service
 *
 *	Authors: JIANG Dan <dan.jiang@orange-ftgroup.com> 
 *	
 *	This software and associated documentation files (the "Software") 
 *	are copyright (C) 2005 LiPS Linux Phone Standards Forum [FranceTelecom] 
 *	All Rights Reserved. 
 *
 *	A copyright license is hereby granted for redistribution and use of 
 *	the Software in source and binary forms, with or without modification, 
 *	provided that the following conditions are met: 
 *	- Redistributions of source code must retain the above copyright notice, 
 *	this copyright license and the following disclaimer. 
 *- Redistributions in binary form must reproduce the above copyright 
 *	notice, this copyright license and the following disclaimer in the 
 *	documentation and/or other materials provided with the distribution. 
 *	- Neither the name of LiPS  nor the names of its Members may be used 
 *	to endorse or promote products derived from the Software without 
 *	specific prior written permission. 
 *
 *	A patent license for any Necessary Claims owned by Members of LiPS Forum 
 *	to make, have made, use, import, offer to sell, lease and sell or otherwise 
 *	distribute any implementation compliant with the any specification adopted 
 *	by the LiPS Forumcan be obtained from the respective Members on reasonable 
 *	and non-discriminatory terms and conditions and under reciprocity, as 
 *	regulated in more detail in the Internal Policy of the LiPS Forum. 
 *
 *	THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDER, ITS MEMBERS AND CONTRIBUTORS 
 *	"AS IS", AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, 
 *	THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE 
 *	AND NON-INFRINGEMENT ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER, 
 *	ITS MEMBERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *	SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, 
 *	PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; 
 *	OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 *	WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 *	ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
 *	POSSIBILITY OF SUCH DAMAGE. 
 */


#include "linestate.h"
#include "connectedwindow.h"
#include "icmc.h"
#include "inputwindow.h"
#include "missedcallwindow.h"
#include "msgdialog.h"
#include <voc.h>
#include <pim.h>
#include <iac.h>


#include <sys/stat.h>


#define PHONE_ADDRESSBOOK_SERVICE "phone.addressbook.service"
#define PHONE_ADDRESSBOOK_OBJECT "/phone/addressbook/service"
#define PHONE_ADDRESSBOOK_INTERFACE "phone.addressbook.service"


#define ADDRESSBOOK_SEARCH_WITH_REPLY "addressbooksearch"
#define ADDRESSBOOK_SEARCH_INVOKE_UI "addressbookinvokeui"

#define ADDRESSBOOK_APP_SERVICE               "ABook"

#define ADDRESSBOOK_DBUS_SERVICE              "app.addressbook.service"
#define ADDRESSBOOK_DBUS_OBJECT               "/app/addressbook/service"
#define ADDRESSBOOK_DBUS_INTERFACE            "app.addressbook.service"

#define ADDRESSBOOK_DBUS_METHOD_MAIN          "voice_find"
#define ADDRESSBOOK_DBUS_METHOD_MULTI_SEL     "mul_select"

#define DEFAULT_PIXMAP_PATH                   PREFIX "/share/voicecall/pixmaps"
#define DEFAULT_AB_DB_PATH                    PREFIX "/share/addressbook/database"

gint
active_call_index ()
{
  int index = 0;

  for (; index < call_array->len; index++)
  {
    if (g_array_index (call_array, call_desc, index).state == ACTIVE
        || (g_array_index (call_array, call_desc, index).state == CONFERENCE
            && !isconf_hold))
      return index;
  }
  if (index == call_array->len)
    return -1;

  return -1;
}

gint
busy_line_count ()
{
  int index = 0, count = 0;

  if (HASCONFERENCE)
    count++;
  for (; index < call_array->len; index++)
  {
    if (g_array_index (call_array, call_desc, index).state == ACTIVE
        || g_array_index (call_array, call_desc, index).state == HOLD)
      count++;
  }
  return count;
}

gint
call_free (gint index)
{

  g_print ("\nin %s,\n", __FUNCTION__);

  if (g_array_index (call_array, call_desc, index).timer != -1)
    g_source_remove (g_array_index (call_array, call_desc, index).timer);
  g_print ("\nin %s,step 1:\n", __FUNCTION__);
  g_print ("\nindex= %d\n", index);
  g_print ("\nnum= %s\n", g_array_index (call_array, call_desc, index).num);
  g_print ("\nname= %s\n", g_array_index (call_array, call_desc, index).num);



/*  g_free(g_array_index(call_array,call_desc,index).num); */
  g_print ("\n111\n");
  g_free (g_array_index (call_array, call_desc, index).name);
  g_print ("\n222\n");

/*   g_free(g_array_index(call_array,call_desc,index).dial_num_in_active); */
  gdk_pixbuf_unref (g_array_index (call_array, call_desc, index).pixbuf);
  g_print ("\n333\n");

  g_print ("\nin %s,step 2:\n", __FUNCTION__);

  g_array_remove_index (call_array, index);
  return TRUE;
}


gint
freeonecall (gint index)
{

  if (g_array_index (call_array, call_desc, index).state != CONFERENCE)
    call_free (index);
  else
  {
    int pos = 0;

    for (; pos < call_array->len; pos++)
    {
      if (g_array_index (call_array, call_desc, pos).state == CONFERENCE)
      {
        call_free (pos);
        pos -= 1;
      }
    }
  }
  return TRUE;

}

gint
init_ui ()
{
  call_array = g_array_new (FALSE, FALSE, sizeof (call_desc));

  pim_init ();
  voch_init ();
  g_print ("Personal Information Management\n\n\n ");
  displaycallnum = -1;
  isspeaker = FALSE;
  ismute = FALSE;
  isconf_hold = FALSE;
  white.pixel = 0;
  white.red = 0xffff;
  white.green = 0xffff;
  white.blue = 0xffff;
  iswifi_flag = FALSE;
  haswifi_flag = FALSE;

  isfirst_flag = TRUE;
  IMAGE_DIR = getenv ("VOC_IMAGE_PATH");
  if (IMAGE_DIR == NULL)
  {
    IMAGE_DIR=DEFAULT_PIXMAP_PATH;
  }
  ABOOK_DATA_PATH = getenv ("ABOOK_DATA_PATH");
  if (ABOOK_DATA_PATH == NULL)
  {
    ABOOK_DATA_PATH=DEFAULT_AB_DB_PATH;
  }
  if (!IMAGE_DIR || !ABOOK_DATA_PATH)
    return FALSE;

  gchar dir[100];

  strcpy (dir, IMAGE_DIR);
  strcat (dir, "/dailing.png");
  if (!load_image (dir, NULL, &bg_pixmap, NULL, NULL))
    return FALSE;
  strcpy (dir, IMAGE_DIR);
  strcat (dir, "/default_caller_optional.png");
  default_photo =
    gdk_pixbuf_new_from_file_at_size (dir, PHOTO_SIZE_W, PHOTO_SIZE_W, NULL);
  if (default_photo == NULL)
  {
    return FALSE;
  }
  return TRUE;
}

static gint
get_indexof_cid (guint cid)
{
  int i = 0;

  for (; i < call_array->len; i++)
  {
    if (g_array_index (call_array, call_desc, i).cid == cid)
      return i;
  }
  return -1;
}

static gboolean
time_add (gpointer cid)
{
  int index = 0;

  if ((index = get_indexof_cid ((gint) cid)) == -1)
    return FALSE;
  g_array_index (call_array, call_desc, index).time++;
//   if(0 == displaycallnum)
  update_time ();
  return TRUE;
}


gint
addnewcall (guint cid, gchar *name, gchar *num, State state,
            GtkWidget *image)
{
  call_desc newcall;

  newcall.cid = cid;
  newcall.state = state;
  newcall.name = name;
  newcall.num = num;
  newcall.time = 0;
  newcall.timer = -1;
  newcall.dial_num_in_active = "";

  if (image)
  {
    newcall.pixbuf = gtk_image_get_pixbuf (GTK_IMAGE (image));
    g_object_ref (newcall.pixbuf);
  }
  else
    newcall.pixbuf = NULL;

  g_array_append_vals (call_array, &newcall, 1);
  if (state == ACTIVE)
  {
    g_array_index (call_array, call_desc, call_array->len - 1).timer =
      g_timeout_add (1000, (GSourceFunc) time_add, (gpointer) cid);
    displaycallnum = ACTIVECALLINDEX;
  }

  g_print ("\ncurrent array size is %d\n", call_array->len);
  return TRUE;
}

gboolean
hasconference ()
{
  gint index = 0;

  for (; index < call_array->len; index++)
  {
    if (g_array_index (call_array, call_desc, index).state == CONFERENCE)
      break;
  }
  if (index == call_array->len)
    return FALSE;
  return TRUE;
}

gint
conf_call_count ()
{
  gint index = 0, count = 0;

  for (; index < call_array->len; index++)
  {
    if (g_array_index (call_array, call_desc, index).state == CONFERENCE)
      count++;
  }
  return count;
}

gint
cb_conference (guint cid)
{
  int index = 0;

  if ((index = get_indexof_cid (cid)) == -1)
    return FALSE;
  isconf_hold = FALSE;
  if (HASCONFERENCE)
  {
    if (g_array_index (call_array, call_desc, index).state == INCOMING)
    {
      g_array_index (call_array, call_desc, index).state = CONFERENCE;
    }
    else
    {
      g_array_index (call_array, call_desc, index).state = CONFERENCE;
    }
  }
  else
  {
    int i = 0;

    for (; i < call_array->len; i++)
    {
      g_array_index (call_array, call_desc, i).state = CONFERENCE;
    }
  }
  displaycallnum = ACTIVECALLINDEX;
  update_list ();
  return TRUE;
}

gint
cb_exitconf (guint cid)
{
  int index = 0;

  if ((index = get_indexof_cid (cid)) == -1)
    return FALSE;
  if (!HASCONFERENCE || BUSYLINECOUNT == 2)
    return FALSE;
  isconf_hold = TRUE;
  g_array_index (call_array, call_desc, index).state = ACTIVE;
  if (conf_call_count () == 1)
  {
    int i = 0;

    for (; i < call_array->len; i++)
    {
      if (g_array_index (call_array, call_desc, i).state == CONFERENCE)
        g_array_index (call_array, call_desc, i).state = HOLD;
    }
  }
  displaycallnum = ACTIVECALLINDEX;
  update_list ();
  update_display ();
  return TRUE;
}


gint
holdcall_bak (guint cid)
{
  g_print ("Enter the holdcall of the ui part,#######################\n");
  if (cid != -1)
  {
    int i = 0;

    if ((i = get_indexof_cid (cid)) == -1)
      return FALSE;
    if (g_array_index (call_array, call_desc, i).state == HOLD)
    {
      g_array_index (call_array, call_desc, i).state = ACTIVE;
    }
    else if (g_array_index (call_array, call_desc, i).state == ACTIVE)
    {
      g_array_index (call_array, call_desc, i).state = HOLD;
    }
    flag_fn = 1;
    g_print
      ("flag_fn=111111111111111111111111111111111111111111111111111111111\n");
    if (get_connectedwindow ()
        && gdk_window_is_visible (get_connectedwindow ()->window))
      update_display ();
    return TRUE;
  }
  return FALSE;
}

static gint
nextcall (gint index)
{
  gint nextpos = 0;

  for (; nextpos < call_array->len; nextpos++)
  {
    if (g_array_index (call_array, call_desc, nextpos).state != CONFERENCE
        && nextpos != index)
      break;
  }
  if (nextpos == call_array->len)
    return -1;
  else
    return nextpos;
}

static gint
findicmc ()
{
  gint icmcindex = 0;

  for (; icmcindex < call_array->len; icmcindex++)
  {
    if (g_array_index (call_array, call_desc, icmcindex).state == INCOMING)
      break;
  }
  if (icmcindex == call_array->len)
    return -1;
  else
    return icmcindex;
}

gint
holdcall (guint cid)
{


  g_print ("Enter the holdcall of the ui part,#######################\n");
  if (cid != -1)
  {
    int i = 0;

    if ((i = get_indexof_cid (cid)) == -1)
      return FALSE;

    if (g_array_index (call_array, call_desc, i).state == CONFERENCE)
    {
      isconf_hold = !isconf_hold;
      if (isconf_hold)
      {
        gint icmcindex = findicmc ();

        if (icmcindex >= 0)
        {
          activecall (g_array_index (call_array, call_desc, icmcindex).cid);
        }
        else if (BUSYLINECOUNT > 1)
        {
          gint nextpos = nextcall (i);

          if (nextpos >= 0)
          {
            g_array_index (call_array, call_desc, nextpos).state = ACTIVE;
          }
        }
      }                         //end of if(is_hold)
      else                      //else if (is_hold)
      {
        if (BUSYLINECOUNT > 1)
        {
          gint nextpos = nextcall (i);

          if (nextpos >= 0)
          {
            g_array_index (call_array, call_desc, nextpos).state = HOLD;
          }
        }
      }
    }


    else if (g_array_index (call_array, call_desc, i).state == HOLD)
    {
      g_array_index (call_array, call_desc, i).state = ACTIVE;
      if (BUSYLINECOUNT > 1)
      {
        if (g_array_index
            (call_array, call_desc,
             (i + 1) % (call_array->len)).state == CONFERENCE)
          isconf_hold = TRUE;
        else
          g_array_index (call_array, call_desc,
                         (i + 1) % (call_array->len)).state = HOLD;
      }

    }
    else if (g_array_index (call_array, call_desc, i).state == ACTIVE)
    {
      g_array_index (call_array, call_desc, i).state = HOLD;
      gint icmcindex = findicmc ();

      if (icmcindex >= 0)
        activecall (g_array_index (call_array, call_desc, icmcindex).cid);
      else if (BUSYLINECOUNT > 1)
      {
        if (g_array_index
            (call_array, call_desc,
             (i + 1) % (call_array->len)).state == CONFERENCE)
          isconf_hold = FALSE;
        else
          g_array_index (call_array, call_desc,
                         (i + 1) % (call_array->len)).state = ACTIVE;
        displaycallnum = ACTIVECALLINDEX;
      }
    }
    flag_fn = 1;
    g_print
      ("flag_fn=111111111111111111111111111111111111111111111111111111111\n");
    if (get_connectedwindow ()
        && gdk_window_is_visible (get_connectedwindow ()->window))
    {

      update_display ();
      update_list ();
    }
    return TRUE;
  }
  return FALSE;
}

gint
swap ()
{
  if (BUSYLINECOUNT < 2)
    return FALSE;
  if (ACTIVECALLINDEX == -1)
    return FALSE;
  g_array_index (call_array, call_desc, ACTIVECALLINDEX).state = HOLD;
//   activelinenum=(activelinenum+1)%(call_array->len);
  g_array_index (call_array, call_desc, ACTIVECALLINDEX).state = ACTIVE;
  displaycallnum = ACTIVECALLINDEX;
  update_list ();
  return TRUE;
}

gint
changestate (gint linenum, State state)
{
  g_array_index (call_array, call_desc, linenum).state = state;
  return TRUE;
}

gint
dropactivecall ()
{
  return 0;
}

gint
dropholdcall ()
{
  int i = 0;

  for (; i < call_array->len; i++)
    if (g_array_index (call_array, call_desc, i).state == HOLD)
      return 0;
  return FALSE;
}

gint
dropcall_cid (guint cid)
{
  static UInt32 missing_flag = 0, pcid;

  pcid = cid;
  if (cid > 60000)
  {
    haswifi_flag = FALSE;
    isfirst_flag = TRUE;
  }

  if (cid > 30000 && cid < 60000)
  {

    cid = cid - 30000;
    missing_flag++;
  }
  else
  {
    //missing_flag=0;
    if (cid == 60008)
      cid = 60001;
  }

  g_print
    ("ENTER the dropcall_id function, cid=%d, index=%d~~~~~~~~~~~~~~~~\n\n\n",
     cid, get_indexof_cid (cid));
  int index = 0;

  if ((index = get_indexof_cid (cid)) == -1)
    return FALSE;

  gboolean is_conf = g_array_index (call_array, call_desc,
                                    index).state == CONFERENCE ? 1 : 0;

  if (g_array_index (call_array, call_desc, index).state == INCOMING)
  {
    icmc_destroy_window ();
    //if (missing_flag>0){
    if (pcid > 30000 && pcid < 60000)
    {
      //missing_window_display();
      voc_call_history_add_history (g_array_index
                                    (call_array, call_desc, index).num,
                                    INCOMING_MISSED, 0);
      show_misscalldialog (&missing_flag);
      g_print ("\t\t\t\t\t\tMISSING CALL\t\t\t!!!!!\n");
    }
    else
    {
      voc_call_history_add_history (g_array_index
                                    (call_array, call_desc, index).num,
                                    INCOMING_ANSWER, 0);
    }
    freeonecall (index);

    return TRUE;
  }                             //end of if==incoming

  GString *temp_result = g_string_new ("End call:");
  gchar *name, *num;

  name = g_array_index (call_array, call_desc, index).name;
  num = g_array_index (call_array, call_desc, index).num;
  if (strcmp (name, "Unknown"))
  {
    g_string_append (temp_result, name);
  }
  else
    g_string_append (temp_result, num);


  if (temp_result->len > 16)
  {
    temp_result = g_string_truncate (temp_result, 13);
    g_string_append (temp_result, "...");
  }

  g_string_append (temp_result, "\n");
  gchar *temp = formattime (g_array_index (call_array, call_desc, index).time);

  g_string_append (temp_result, temp);
  g_free (temp);
  create_msgdialog (temp_result->str, MSG_HANGUP);
  g_string_free (temp_result, TRUE);
  //JD
  int j = 0;

  g_print ("before call_free, len=%d\n", call_array->len);
  for (j = 0; j < call_array->len; j++)
  {
    g_print ("STEP 1:\t\t\tis conference, index=%d, status=%d\n\n\n", j,
             g_array_index (call_array, call_desc, j).state);
  }
  //JDD


  call_free (index);
  //JD
  g_print ("after call_free, len=%d\n", call_array->len);
  g_print ("before call_free, len=%d\n", call_array->len);
  for (j = 0; j < call_array->len; j++)
  {
    g_print ("STEP 1.1:\t\t\tis conference, index=%d, status=%d\n\n\n", j,
             g_array_index (call_array, call_desc, j).state);
  }
  //JDD

  if (is_conf)
  {
    g_print ("STEP 1.2:\t\t\tis conference, conf count=%d\n\n\n",
             conf_call_count ());
    if (conf_call_count () == 1)
    {
      int i = 0;

      for (; i < call_array->len; i++)
      {
        g_print ("STEP 1.3:\t\t\tis conference, index=%d, status=%d\n\n\n",
                 i, g_array_index (call_array, call_desc, i).state);
        if (g_array_index (call_array, call_desc, i).state == CONFERENCE)
        {
          if (isconf_hold)
            g_array_index (call_array, call_desc, i).state = HOLD;
          else
            g_array_index (call_array, call_desc, i).state = ACTIVE;
        }
      }
    }
  }

  else if (BUSYLINECOUNT >= 1)
  {
    g_print ("STEP 2:\t\t\tis BUSYLINECOUNT=%d\n\n\n", BUSYLINECOUNT);
    g_print ("STEP 2:\t\t\tistate0=%d\n\n\n",
             g_array_index (call_array, call_desc, 0).state);

    if (g_array_index (call_array, call_desc, 0).state == CONFERENCE)
    {
      isconf_hold = FALSE;
    }
    else
    {
      if ((pcid < 30000) && !haswifi_flag)
        g_array_index (call_array, call_desc, 0).state = ACTIVE;
    }
  }                             //end of else if busy
  else
  {
    //gtk_widget_hide_all(get_connectedwindow());
    g_print ("STEP 3:\t\t\tOthers\n\n\n");

    gtk_window_iconify (GTK_WINDOW (get_connectedwindow ()));
//jd



    if (get_missedwindow ())
    {                           //gtk_window_iconify(GTK_WINDOW(get_callingwindow()));
      gtk_widget_show (get_missedwindow ());
    }

    return TRUE;
  }

  g_print ("STEP 4:\t\tdisplay=%d,active=%d, state0=%d\n\n\n",
           displaycallnum, ACTIVECALLINDEX, g_array_index (call_array,
                                                           call_desc, 0).state);
  if (ACTIVECALLINDEX > -1)
    displaycallnum = ACTIVECALLINDEX;
  else
    displaycallnum = 0;
  update_list ();
  update_display ();
  g_print ("STEP 5:the dropcall_id function~~~~~~~~~~~~~~~~\n\n\n");

  return TRUE;
}


gint
dropcall_all (guint cid)
{

  g_print ("DROP ALL!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!\n\n\n");

  int index = 0;

  if ((index = get_indexof_cid (cid)) == -1)
    return FALSE;
  UInt32 pos;
  GString *temp_result = g_string_new ("End call:");
  gchar *name;


  if (temp_result->len > 16)
  {
    temp_result = g_string_truncate (temp_result, 13);
    g_string_append (temp_result, "...");
  }

  g_string_append (temp_result, "\n");
  gchar *temp = formattime (g_array_index (call_array, call_desc, index).time);

  g_string_append (temp_result, temp);
  g_free (temp);
  create_msgdialog (temp_result->str, MSG_HANGUP);
  g_string_free (temp_result, TRUE);


  for (pos = 0; pos < call_array->len; pos++)
  {
    call_free (pos);
    pos -= 1;
  }
  gtk_window_iconify (GTK_WINDOW (get_connectedwindow ()));

  if (get_missedwindow ())
  {                             //gtk_window_iconify(GTK_WINDOW(get_callingwindow()));
    gtk_widget_show (get_missedwindow ());
  }

  return TRUE;


}


gint
dropcall_active_cid (guint cid)
{
  g_print ("ENter replace hold/active callback=========================\n");
  int index = 0;

  if ((index = get_indexof_cid (cid)) == -1)
    return FALSE;

  /* add call end dialog ----------6_21 */

  GString *temp_result = g_string_new ("End call:");
  gchar *name, *num;

  if (g_array_index (call_array, call_desc, index).state == CONFERENCE)
    name = "Conference";
  else
    name = g_array_index (call_array, call_desc, index).name;
  num = g_array_index (call_array, call_desc, index).num;
  if (strcmp (name, "Unknown"))
  {
    g_string_append (temp_result, name);
  }
  else
    g_string_append (temp_result, num);


  if (temp_result->len > 16)
  {
    temp_result = g_string_truncate (temp_result, 13);
    g_string_append (temp_result, "...");
  }

  g_string_append (temp_result, "\n");
  gchar *temp = formattime (g_array_index (call_array, call_desc, index).time);

  g_string_append (temp_result, temp);
  g_free (temp);
  create_msgdialog (temp_result->str, MSG_HANGUP);
  g_string_free (temp_result, TRUE);



  freeonecall (index);
  gint icmcindex = findicmc ();

  if (icmcindex != -1)
  {
    gint pos;

    for (pos = 0; pos < call_array->len; pos++)
    {
      if (g_array_index (call_array, call_desc, pos).state == ACTIVE)
      {
        g_array_index (call_array, call_desc, pos).state = HOLD;
        g_print ("Change state%s\n\n\n",
                 g_array_index (call_array, call_desc, pos).num);
        break;
      }
      else if (g_array_index (call_array, call_desc, pos).state == CONFERENCE)
      {
        isconf_hold = TRUE;
        break;
      }
    }
    activecall (g_array_index (call_array, call_desc, icmcindex).cid);
  }
  else if (BUSYLINECOUNT >= 1)
  {
    if (g_array_index (call_array, call_desc, 0).state == CONFERENCE)
    {
      isconf_hold = FALSE;
    }
    else
    {
      g_array_index (call_array, call_desc, 0).state = ACTIVE;
    }
  }
  else
  {
    gtk_window_iconify (GTK_WINDOW (get_connectedwindow ()));
    if (get_missedwindow ())
    {                           //gtk_window_iconify(GTK_WINDOW(get_callingwindow()));
      gtk_widget_show (get_missedwindow ());
    }

    return TRUE;
  }

  displaycallnum = ACTIVECALLINDEX;
  update_list ();
  update_display ();
  return TRUE;
}



gchar *
statetostring (State state)
{
  switch (state)
  {
    case FREE:
      return "Free";
    case CALLING:
      return "Calling";
    case ACTIVE:
      return "Active";
    case HOLD:
      return "Hold";
    case CONFERENCE:
      return "Conference";
    default:
      return "";
  }
}

gint
activecall (guint cid)
{
  int index = 0;

  if ((index = get_indexof_cid (cid)) == -1)
    return FALSE;
  if (g_array_index (call_array, call_desc, index).state == INCOMING)
  {
    //busylinecount++;
    g_array_index (call_array, call_desc, index).state = ACTIVE;
    //activelinenum = index ;
    displaycallnum = ACTIVECALLINDEX;
    show_connectedwindow ();
    icmc_destroy_window ();
    g_array_index (call_array, call_desc, index).timer =
      g_timeout_add (1000, (GSourceFunc) time_add, (gpointer) cid);


    voc_call_history_add_history (g_array_index
                                  (call_array, call_desc, index).num,
                                  INCOMING_ANSWER, 0);

    return TRUE;
  }
  g_array_index (call_array, call_desc, index).state = ACTIVE;
  displaycallnum = ACTIVECALLINDEX;
  if (gdk_window_is_visible (get_connectedwindow ()->window))
  {
    update_display ();
    update_list ();
  }
  return TRUE;
}

gboolean
invoke_addressbook_ui_iac ()
{
  extern iac_context_t *context;

//   iac_context_t *context = NULL;
  iac_arguments_t *args = NULL;
  iac_arguments_t *reply = NULL;

  if (!context)
    context = iac_initialize ();

  if (iac_start_service (context,
                         ADDRESSBOOK_APP_SERVICE,
                         ADDRESSBOOK_DBUS_METHOD_MAIN) != IAC_OK)
  {
    g_print ("%s(): start app service %s failure\n",
             __FUNCTION__, ADDRESSBOOK_APP_SERVICE);

    return EXIT_FAILURE;
  }
  iac_execute_service (context,
                       ADDRESSBOOK_DBUS_SERVICE,
                       ADDRESSBOOK_DBUS_OBJECT,
                       ADDRESSBOOK_DBUS_INTERFACE,
                       ADDRESSBOOK_DBUS_METHOD_MAIN, args, reply);

  iac_arguments_free (args);
  iac_arguments_free (reply);


  return EXIT_SUCCESS;
}

/* 
gboolean invoke_addressbook_ui()
{
  DBusError error;
  DBusConnection *bus;
  dbus_error_init (&error);
  bus= dbus_bus_get (DBUS_BUS_SESSION, &error);
  DBusMessage *message;
  message = dbus_message_new_method_call (PHONE_ADDRESSBOOK_SERVICE,
                                          PHONE_ADDRESSBOOK_OBJECT,
                                          PHONE_ADDRESSBOOK_INTERFACE,
                                          ADDRESSBOOK_SEARCH_INVOKE_UI);
  if (message == NULL)
  {
    return FALSE;
  }
  dbus_message_set_no_reply (message, TRUE);
  
  dbus_connection_send (bus, message, NULL);
        
  dbus_message_unref (message);
  return TRUE;
}
*/

gboolean
search_from_addressbook (const gchar *num, gchar **name_b,
                         gchar **photo_path_b, gchar **Ringtone_path_b)
{
  ab_contact_t *temp = NULL;
  ab_contact_t *result = NULL;
  ab_uuid_t **uids = NULL;
  ab_sid_t sid;
  ab_err_t err;
  gint i;
  gint databaseid = 0;

  if (!uids)
  {
    databaseid = 0;
    err = ab_connect (databaseid, &sid);
    if (err != AB_ERROR_NONE)
    {
      g_print ("%s(): connecting to %d failed.\n", __FUNCTION__, 0);
      pim_quit ();
      return EXIT_FAILURE;
    }
    int field = 3;

    for (; field < 7; field++)
    {
      ab_create_contact (sid, &temp);
      ab_contact_set_field (temp, field, num);
      uids = ab_get_contact_ids (sid, temp);
      if (uids)
      {
        ab_contact_free (temp);
        break;
      }
      ab_contact_free (temp);
      ab_create_contact (sid, &temp);

      if (strstr (num, "+86"))
      {
        ab_contact_set_field (temp, field, num + 3);
      }
      else
      {
        gchar *num_temp = g_strdup_printf ("+86%s", num);

        ab_contact_set_field (temp, field, num_temp);
        g_free (num_temp);
      }
//       gchar num_temp[50];
//       strcpy(num_temp,"+86");
//       strcat(num_temp,num);
//       ab_contact_set_field(temp,field,num_temp);
      uids = ab_get_contact_ids (sid, temp);
      if (uids)
      {
        ab_contact_free (temp);
        break;
      }
    }
  }

  if (!uids)
  {
    databaseid = 1;
    ab_disconnect (sid);
    err = ab_connect (databaseid, &sid);
    if (err != AB_ERROR_NONE)
    {
      g_print ("%s(): connecting to %d failed.\n", __FUNCTION__, 0);
      pim_quit ();
      return EXIT_FAILURE;
    }
    ab_create_contact (sid, &temp);
    ab_contact_set_field (temp, 5, num);
    uids = ab_get_contact_ids (sid, temp);
    ab_contact_free (temp);

    if (!uids)
    {


      ab_create_contact (sid, &temp);
      if (strstr (num, "+86"))
      {
        ab_contact_set_field (temp, 5, num + 3);
      }
      else
      {
        gchar *num_temp = g_strdup_printf ("+86%s", num);

        ab_contact_set_field (temp, 5, num);
        g_free (num_temp);
      }
      uids = ab_get_contact_ids (sid, temp);
      ab_contact_free (temp);
    }
  }



  if (uids)
  {
    if (databaseid == 0)
    {
      for (i = 0; uids[i] != NULL; i++)
      {
        gchar *name, *photo_path_t, *Ringtone_path;

        ab_get_contact (sid, uids[0], &result);
        ab_contact_get_field (result, 2, (void *) &name);
        ab_contact_get_field (result, 11, (void *) &photo_path_t);
        ab_contact_get_field (result, 11, (void *) &Ringtone_path);

        if (photo_path_b)
          *photo_path_b =
            g_strdup_printf ("%s/pixmaps/%s", ABOOK_DATA_PATH, photo_path_t);
        if (name_b)
          *name_b = g_strdup (name);
        if (Ringtone_path_b)
          *Ringtone_path_b =
            g_strdup_printf ("%s/%s", ABOOK_DATA_PATH, photo_path_t);
        ab_contact_free (result);
        break;
      }
    }
    else
    {
      for (i = 0; uids[i] != NULL; i++)
      {
        gchar *name;

        ab_get_contact (sid, uids[0], &result);
        ab_contact_get_field (result, 4, (void *) &name);
        if (photo_path_b)
          *photo_path_b =
            g_strdup_printf ("%s/pixmaps/%s", ABOOK_DATA_PATH, "iconsim.png");
        if (name_b)
          *name_b = g_strdup (name);
        if (Ringtone_path_b)
          *Ringtone_path_b = g_strdup_printf ("%s/%s", ABOOK_DATA_PATH, "");
        ab_contact_free (result);
        break;
      }
    }

    for (i = 0; uids[i] != NULL; i++)
    {
      g_free (uids[i]);
    }

    g_free (uids);
  }
  else
  {
    if (photo_path_b)
      *photo_path_b = NULL;
    if (name_b)
      *name_b = NULL;
    if (Ringtone_path_b)
      *Ringtone_path_b = NULL;
  }

  ab_disconnect (sid);

//   pim_quit();

  return EXIT_SUCCESS;
}




gchar *
formattime (glong time)
{
  gint hour, minute, second;
  gchar *time_s;

  hour = time / 60 / 60;
  minute = time / 60 % 60;
  second = time % 60;
  time_s = g_strdup_printf ("Time:   %02d:%02d:%02d", hour, minute, second);
  return time_s;
}

void
dial_in_active (gchar *num)
{
  gint activecallindex = ACTIVECALLINDEX;

  if (activecallindex == -1)
    return;
//  voc_gsm_send_dtmf (0,
  voc_send_dtmf (0,
                      g_array_index (call_array, call_desc,
                                    activecallindex).cid, num);
  g_print ("\n\n\nThe dtmf send out is %s", num);
  GString *temp =
    g_string_new (g_array_index (call_array, call_desc, activecallindex).
                  dial_num_in_active);
  g_string_append (temp, num);
  g_array_index (call_array, call_desc,
                 activecallindex).dial_num_in_active = temp->str;
  g_string_free (temp, FALSE);
  update_display ();
}

gboolean
load_image (const gchar *filename, GdkPixbuf **pixbuf_b,
            GdkPixmap **pixmap_b, gint *width_b, gint *height_b)
{
  GtkWidget *window;
  GdkPixbuf *pbuf;
  GdkPixmap *pixmap;
  gint width, height;

  pbuf = gdk_pixbuf_new_from_file (filename, NULL);
  if (pbuf == NULL)
  {
    return FALSE;
  }
  width = gdk_pixbuf_get_width (pbuf);
  height = gdk_pixbuf_get_height (pbuf);

  window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
  gtk_widget_realize (window);

  pixmap = gdk_pixmap_new (window->window, width, height, -1);
  gdk_draw_rectangle (pixmap, window->style->white_gc, TRUE, 0, 0, width,
                      height);
  gdk_draw_pixbuf (pixmap, NULL, pbuf, 0, 0, 0, 0, -1, -1,
                   GDK_RGB_DITHER_NORMAL, 1, 1);
  if (pixbuf_b)
    *pixbuf_b = pbuf;
  else
    gdk_pixbuf_unref (pbuf);
  if (pixmap_b)
    *pixmap_b = pixmap;
  else
    g_object_ref (pixmap);
  if (width_b)
    *width_b = width;
  if (height_b)
    *height_b = height;
  return TRUE;
}

gboolean
popup_menu_key_press_handler (GtkWidget *menu, GdkEventKey *k,
                              gpointer userdata)
{
  switch (k->keyval)
  {
    case GDK_F3:
      g_signal_emit_by_name (menu, "activate-current");
      return TRUE;
    case GDK_F4:
      gtk_menu_popdown (GTK_MENU (menu));
      return TRUE;
    default:
      break;
  }
  return FALSE;
}

gboolean
voc_call_history_add_history (gchar *num, voch_call_type_t type,
                              time_t duration)
{
  voch_callhistory_t *history;
  voch_sid_t sid;

  g_print ("in call history 1\n");

  voch_connect (0, &sid);
  g_print ("in call history 2\n");

  voch_create_history (sid, &history);

  g_print ("in call history 2\n");


  voch_callhistory_set_phonenumber (history, num);

  g_print ("in call history 3\n");

  voch_callhistory_set_duration (history, duration);
  voch_callhistory_set_date (history, time (NULL) - duration);
  voch_callhistory_set_type (history, type);
  voch_add_history (sid, history);
  voch_callhistory_free (history);
  voch_disconnect (sid);
  return TRUE;
}

/* 
vi:ts=2:nowrap:ai:expandtab
*/

