/* 	appVOC - Application of voice call service
 *
 *	Authors: JIANG Dan <dan.jiang@orange-ftgroup.com> 
 *	
 *	This software and associated documentation files (the "Software") 
 *	are copyright (C) 2005 LiPS Linux Phone Standards Forum [FranceTelecom] 
 *	All Rights Reserved. 
 *
 *	A copyright license is hereby granted for redistribution and use of 
 *	the Software in source and binary forms, with or without modification, 
 *	provided that the following conditions are met: 
 *	- Redistributions of source code must retain the above copyright notice, 
 *	this copyright license and the following disclaimer. 
 *- Redistributions in binary form must reproduce the above copyright 
 *	notice, this copyright license and the following disclaimer in the 
 *	documentation and/or other materials provided with the distribution. 
 *	- Neither the name of LiPS  nor the names of its Members may be used 
 *	to endorse or promote products derived from the Software without 
 *	specific prior written permission. 
 *
 *	A patent license for any Necessary Claims owned by Members of LiPS Forum 
 *	to make, have made, use, import, offer to sell, lease and sell or otherwise 
 *	distribute any implementation compliant with the any specification adopted 
 *	by the LiPS Forumcan be obtained from the respective Members on reasonable 
 *	and non-discriminatory terms and conditions and under reciprocity, as 
 *	regulated in more detail in the Internal Policy of the LiPS Forum. 
 *
 *	THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDER, ITS MEMBERS AND CONTRIBUTORS 
 *	"AS IS", AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, 
 *	THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE 
 *	AND NON-INFRINGEMENT ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER, 
 *	ITS MEMBERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *	SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, 
 *	PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; 
 *	OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 *	WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 *	ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
 *	POSSIBILITY OF SUCH DAMAGE. 
 */

#include "linestate.h"
static gint timerid = -1;
static gint times_to_disappear = 4;
static GtkWidget *label;
static GtkWidget *selfwindow;
static msg_type current_type;
static void
softkey_activated (GtkWidget *window, gint nr)
{
  if (nr == 0)
  {
    if (timerid != -1)
      g_source_remove (timerid);
    gtk_widget_destroy (window);
    selfwindow = NULL;
  }
}

static gboolean
setup_softkeys (GtkWidget *window)
{

  if (gpe_skwindow_get_softkey_count (GPE_SKWINDOW (window)) == 2)
  {
    gpe_skwindow_set_softkey_label (GPE_SKWINDOW (window), SK_LEFT, _("OK"));
    gpe_skwindow_set_softkey_label (GPE_SKWINDOW (window), SK_RIGHT, _(""));
    return TRUE;
  }
  return FALSE;
}

static gboolean
dialog_action (GtkWidget *window)
{
  switch (current_type)
  {
    case MSG_NORMAL_TIMEOUT:
      if (timerid != -1)
        g_source_remove (timerid);
      gtk_widget_destroy (window);
      selfwindow = NULL;
      return FALSE;
    case MSG_HANGUP:
      times_to_disappear--;
      if (times_to_disappear <= 0)
      {
        if (timerid != -1)
          g_source_remove (timerid);
        gtk_widget_destroy (window);
        selfwindow = NULL;
        return FALSE;
      }
      else if (times_to_disappear % 3)
        gtk_widget_show (label);
      else
        gtk_widget_hide (label);
      return TRUE;
    default:
      break;
  }
  return TRUE;
}

static gboolean
window_key_press_event (GtkWidget *widget, GdkEventKey *k, gpointer data)
{

  switch (k->keyval)
  {
    case KEY_END:
      softkey_activated (widget, 0);
      return TRUE;
    default:
      break;
  }
  return TRUE;
}

GtkWidget *
create_msgdialog (gchar *msg, msg_type type)
{
  if (selfwindow)
  {
    if (timerid != -1)
      g_source_remove (timerid);
    g_print ("\nready window destroyed\n");
    gtk_widget_destroy (selfwindow);
    g_print ("\nthe window destroyed\n");
    selfwindow = NULL;
  }
  GtkWidget *mainwindow;
  GtkWidget *mainbox;
  GtkWidget *image;
  PangoFontDescription *font_desc;

  current_type = type;

  g_print ("\nthe window create\n");
  mainwindow = gpe_skwindow_new ();
  selfwindow = mainwindow;
  gtk_window_set_type_hint (GTK_WINDOW (mainwindow),
                            GDK_WINDOW_TYPE_HINT_SPLASHSCREEN);

  gtk_window_set_modal (GTK_WINDOW (mainwindow), TRUE);
  gchar dir[100];

  strcpy (dir, IMAGE_DIR);
  strcat (dir, "/pop_icon_info.png");
  image = gtk_image_new_from_file (dir);

  gtk_window_set_position (GTK_WINDOW (mainwindow), GTK_WIN_POS_CENTER);
  gtk_window_set_title (GTK_WINDOW (mainwindow), "Massage");
  gtk_window_set_default_size (GTK_WINDOW (mainwindow), gdk_screen_width (),
                               50);
  GdkColor list_bg = { 0, 57015, 61602, 62879 };
  gtk_widget_modify_bg (mainwindow, GTK_STATE_NORMAL, &list_bg);
  gtk_window_move (GTK_WINDOW (mainwindow), 0, gdk_screen_height () - 73);
  label = gtk_label_new (msg);
  font_desc = pango_font_description_from_string ("Sans Italic 10");
  gtk_widget_modify_font (label, font_desc);
  pango_font_description_free (font_desc);
  gtk_label_set_justify (GTK_LABEL (label), GTK_JUSTIFY_CENTER);

  mainbox = gtk_hbox_new (FALSE, 0);
  gtk_box_pack_start (GTK_BOX (mainbox), image, FALSE, FALSE, 0);
  gtk_box_pack_start (GTK_BOX (mainbox), label, TRUE, TRUE, 0);
  gtk_container_add (GTK_CONTAINER (mainwindow), mainbox);
  g_signal_connect (G_OBJECT (mainwindow), "softkey_activated",
                    G_CALLBACK (softkey_activated), NULL);
  g_signal_connect (G_OBJECT (mainwindow), "key_press_event",
                    G_CALLBACK (window_key_press_event), NULL);
  g_print ("\ndebug 1: the window create\n");

  if (type == MSG_HANGUP)
  {
    times_to_disappear = 10;
    timerid = g_timeout_add (350, (GSourceFunc) dialog_action, mainwindow);
  }

  g_print ("\ndebug 2: the window create\n");

  if (type == MSG_NORMAL_TIMEOUT)
    timerid = g_timeout_add (2000, (GSourceFunc) dialog_action, mainwindow);

  g_print ("\ndebug 3: the window create\n");

  setup_softkeys (mainwindow);
  gtk_widget_show_all (mainwindow);
  return mainwindow;
}


/* 
vi:ts=2:nowrap:ai:expandtab
*/

