/* 	appVOC - Application of voice call service
 *
 *	Authors: JIANG Dan <dan.jiang@orange-ftgroup.com> 
 *	
 *	This software and associated documentation files (the "Software") 
 *	are copyright (C) 2005 LiPS Linux Phone Standards Forum [FranceTelecom] 
 *	All Rights Reserved. 
 *
 *	A copyright license is hereby granted for redistribution and use of 
 *	the Software in source and binary forms, with or without modification, 
 *	provided that the following conditions are met: 
 *	- Redistributions of source code must retain the above copyright notice, 
 *	this copyright license and the following disclaimer. 
 *- Redistributions in binary form must reproduce the above copyright 
 *	notice, this copyright license and the following disclaimer in the 
 *	documentation and/or other materials provided with the distribution. 
 *	- Neither the name of LiPS  nor the names of its Members may be used 
 *	to endorse or promote products derived from the Software without 
 *	specific prior written permission. 
 *
 *	A patent license for any Necessary Claims owned by Members of LiPS Forum 
 *	to make, have made, use, import, offer to sell, lease and sell or otherwise 
 *	distribute any implementation compliant with the any specification adopted 
 *	by the LiPS Forumcan be obtained from the respective Members on reasonable 
 *	and non-discriminatory terms and conditions and under reciprocity, as 
 *	regulated in more detail in the Internal Policy of the LiPS Forum. 
 *
 *	THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDER, ITS MEMBERS AND CONTRIBUTORS 
 *	"AS IS", AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, 
 *	THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE 
 *	AND NON-INFRINGEMENT ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER, 
 *	ITS MEMBERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *	SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, 
 *	PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; 
 *	OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 *	WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 *	ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
 *	POSSIBILITY OF SUCH DAMAGE. 
 */

#include <glib.h>
#include <stdio.h>
#include <stdlib.h>
#include <voc.h>

#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>


#include "linestate.h"
#include "inputwindow.h"
#include "callingwindow.h"

#include "callbacks.h"

#include "shared.h"

#include <iac.h>


/* Another instance started and has passed some state to us.  */
/* 
static void
handoff_callback (Handoff *handoff, char *data)
{
  show_inputwindow ("", NULL, NULL);
}
*/

iac_context_t *context = NULL;
static gboolean
cb_calling (iac_arguments_t *arguments, gpointer user_data)
{

  gchar *peer_number = NULL;
  gchar *peer_name = NULL;
  gchar *peer_photopath = NULL;
  gint i;
  gint count;

  count = iac_arguments_size (arguments);
  if (count > 3)
    return IAC_ERROR;
  for (i = 0; i < count; i++)
  {
    iac_value_t *val = iac_arguments_get_arg (arguments, i);

    if (val->vtype != IAC_TYPE_STRING)
      return IAC_ERROR;
    else
    {
      switch (i)
      {
        case 0:
          peer_number = g_strdup (val->value.s);
          break;
        case 1:
          peer_name = g_strdup (val->value.s);
          break;
        case 2:
          peer_photopath = g_strdup (val->value.s);
          break;
        default:
          break;
      }
    }
  }
  if (!peer_number)
    return IAC_ERROR;
  if (!peer_name && !peer_photopath)
    search_from_addressbook (peer_number, &peer_name, &peer_photopath, NULL);
  show_callingwindow (peer_number, peer_name, peer_photopath);
  return IAC_OK;
}

static gboolean
cb_num_return (iac_arguments_t *arguments, gpointer user_data)
{

  gchar *peer_number = NULL;
  gchar *peer_name = NULL;
  gchar *peer_photopath = NULL;
  gint i;
  gint count;

  count = iac_arguments_size (arguments);
  if (count > 3)
    return IAC_ERROR;
  for (i = 0; i < count; i++)
  {
    iac_value_t *val = iac_arguments_get_arg (arguments, i);

    if (val->vtype != IAC_TYPE_STRING)
      return IAC_ERROR;
    else
    {
      switch (i)
      {
        case 0:
          peer_number = g_strdup (val->value.s);
          break;
        case 1:
          peer_name = g_strdup (val->value.s);
          break;
        case 2:
          peer_photopath = g_strdup (val->value.s);
          break;
        default:
          break;
      }
    }
  }
  if (!strcmp (peer_number, " "))
  {
    g_free (peer_number);
    peer_number = "";
  }
  if (!peer_number)
    return IAC_ERROR;
  show_inputwindow (peer_number, peer_name, peer_photopath);
  return IAC_OK;
}

static gboolean
cb_ui_action (const gchar *interface,
              const gchar *method,
              iac_arguments_t *arguments,
              iac_arguments_t *replay, gpointer user_data)
{


  g_print ("\n\naaaaaaaaaaaaa_bbbbbbbbbbbbb_cccccccccccc_dddddddd\n\n");


  if (!strcmp (method, VOC_APP_METHOD_CALLING_INVOKE))
    return cb_calling (arguments, user_data);
  else if (!strcmp (method, VOC_APP_METHOD_NUM_RETURN))
    return cb_num_return (arguments, user_data);
  else
    return IAC_ERROR;
}

/* to write a lock on a flag file
   return 0 means OK
   return -1 means something wrong

   The Voc application will create a lock file when started, the file will
   exist in the system when Voc is running until Voc exits. Applauncher tests
   the existence of this file to determine whether Voc is running (return 1)
   or not (return 0)
*/
static int
accquire_lock (const gchar *lock_file)
{
  int fd;
  int status;
  struct flock lock_it;

  fd = open (lock_file, O_WRONLY | O_CREAT, 600);
  if (fd < 0)
  {
    printf ("open lock file failed\n");
    return -1;
  }

  lock_it.l_type = F_WRLCK;
  lock_it.l_whence = SEEK_SET;
  lock_it.l_start = 0;
  lock_it.l_len = 0;

  if (fcntl (fd, F_SETLK, &lock_it) < 0)
  {
    printf ("set lock error\n");
    return -1;
  }

  return 0;
}



int
main (int argc, char *argv[])
{


  /*GMainLoop  *main_loop = NULL; */

  if (accquire_lock ("/tmp/.voc.pid") != 0)
    return -1;

  g_type_init ();


  context = iac_initialize ();
  if (context == NULL)
  {
    g_print ("%s(): iac_context initialization failure\n", __FUNCTION__);

    return EXIT_FAILURE;
  }

  if (iac_register_service (context, PHONE_VOICE_SERVICE) == IAC_ERROR)
  {
    g_print ("%s(): cannot register service\n", __FUNCTION__);

    return EXIT_FAILURE;
  }


  voc_sid_t sid = 0;

  voc_initialize ();

  g_print ("!!!!!!!!!!!!!!!!after voc_initialize!!!!!!!!!!!!!!!!!!\n\n\n");

  g_thread_init (NULL);

  voc_open_session (0, &sid);

  g_print ("!!!!!!!!!!!!!!!!after open session 1!!!!!!!!!!!!!!!!!!\n\n\n");


  voc_open_session (1, &sid);

  g_print ("!!!!!!!!!!!!!!!!after open session 2!!!!!!!!!!!!!!!!!!\n\n\n");



  if (iac_register_listener (context,
                             PHONE_VOICE_OBJECT,
                             PHONE_VOICE_INTERFACE,
                             cb_ui_action, NULL) != IAC_OK)
  {
    iac_deinitialize (context);
    return EXIT_FAILURE;
  }

  sid = 0;
  g_print ("sid=%d\n\n\n", sid);
/*   change the UI, etc. */
  int i;
  voc_lid_t lid;
  voc_err_t con_err;

  voc_evt_register (0, VOC_EVT_CALL_REMOTE_STATUS, NULL, voc_cb_hungup, NULL);
  voc_evt_register (1, VOC_EVT_CALL_REMOTE_STATUS, NULL, voc_cb_hungup, NULL);
  voc_evt_register (1, VOC_EVT_EXCEPTION, NULL, handler_exception, NULL);


  for (i = 0;; i++)
  {
    con_err = voc_line_connect (sid, i, &lid);

    if (con_err == VOC_ERR_NOT_EXIST)
      continue;
    else
    {
      if (con_err == VOC_ERR_TOO_BIG)
        break;
      else
      {
        //TO do: Connect the line
        voc_line_evt_register (0, lid, VOC_EVT_LINE_CALL, NULL,
                               voc_cb_incoming, NULL);
        voc_line_evt_register (1, lid, VOC_EVT_LINE_CALL, NULL,
                               voc_cb_incoming, NULL);


        char *name, *descp, *local;
        voc_line_signal_t *signal;

        voc_line_get_name (sid, lid, &name);
        voc_line_get_description (sid, lid, &descp);
        voc_line_get_local_address (sid, lid, &local);
        voc_line_get_signal (sid, lid, signal);
      }
    }
  }
  g_print ("sid=%d\n\n\n", sid);

  if (gpe_application_init (&argc, &argv) == FALSE)
  {
    g_print ("Enter gpe-init\n");
    exit (1);
  }
  if (init_ui () == FALSE)
  {
    g_print
      ("init error! \n maybe not find the env:VOC_IMAGE_PATH,ABOOK_DATA_PATH;\nplease specify this two path!");
    exit (1);
  }

  /* 

     Handoff *handoff = handoff_new ();
     const char *home = g_get_home_dir ();
     #define RENDEZ_VOUS "/.gpe-calendar-rendezvous"
     char *rendez_vous = alloca (strlen (home) + strlen (RENDEZ_VOUS) + 1);
     sprintf (rendez_vous, "%s" RENDEZ_VOUS, home);

     g_signal_connect (G_OBJECT (handoff), "handoff",
     G_CALLBACK (handoff_callback), NULL);

     if (handoff_handoff (handoff, rendez_vous, NULL,
     FALSE, NULL, NULL))
     exit (0);
   */

  if (argc == 1)
    show_inputwindow ("", NULL, NULL);
  else
  {
    g_print ("\ncreate_and_hide\n");
    show_inputwindow_hide ("", NULL, NULL);
  }

/* 
  if (argc == 3 && !strcmp(argv[1], "dial")) { 
    printf ("argv[1] is %s, argv[2] is %s\n", argv[1], argv[2]);
    show_inputwindow (argv[2], NULL, NULL);
  }
  */
  g_print ("Enter input window\n");

  gtk_main ();
  return 0;
}

/* 
vi:ts=2:nowrap:ai:expandtab
*/

