/* 	appVOC - Application of voice call service
 *
 *	Authors: JIANG Dan <dan.jiang@orange-ftgroup.com> 
 *	
 *	This software and associated documentation files (the "Software") 
 *	are copyright (C) 2005 LiPS Linux Phone Standards Forum [FranceTelecom] 
 *	All Rights Reserved. 
 *
 *	A copyright license is hereby granted for redistribution and use of 
 *	the Software in source and binary forms, with or without modification, 
 *	provided that the following conditions are met: 
 *	- Redistributions of source code must retain the above copyright notice, 
 *	this copyright license and the following disclaimer. 
 *- Redistributions in binary form must reproduce the above copyright 
 *	notice, this copyright license and the following disclaimer in the 
 *	documentation and/or other materials provided with the distribution. 
 *	- Neither the name of LiPS  nor the names of its Members may be used 
 *	to endorse or promote products derived from the Software without 
 *	specific prior written permission. 
 *
 *	A patent license for any Necessary Claims owned by Members of LiPS Forum 
 *	to make, have made, use, import, offer to sell, lease and sell or otherwise 
 *	distribute any implementation compliant with the any specification adopted 
 *	by the LiPS Forumcan be obtained from the respective Members on reasonable 
 *	and non-discriminatory terms and conditions and under reciprocity, as 
 *	regulated in more detail in the Internal Policy of the LiPS Forum. 
 *
 *	THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDER, ITS MEMBERS AND CONTRIBUTORS 
 *	"AS IS", AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, 
 *	THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE 
 *	AND NON-INFRINGEMENT ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER, 
 *	ITS MEMBERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *	SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, 
 *	PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; 
 *	OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 *	WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 *	ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
 *	POSSIBILITY OF SUCH DAMAGE. 
 */


#include "linestate.h"
#include "volumebar.h"
#include "msgdialog.h"
#include <sys/soundcard.h>
#include <fcntl.h>
static struct
{
  GtkWidget *volumebar;
  GtkWidget *image_roller;
  GtkWidget *image_loudspeaker;
  GtkWidget *image_handset;
  gint roller_pos_x;
  gint roller_pos_y;
  gint left_bound;
  gint right_bound;
  gint step;
  GtkWidget *image_bg;
  GdkGC *gc;
  GdkPixmap *pixmap;
  GdkPixbuf *pixbuf_bg;

} self;
GtkWidget *
getvolumebar ()
{
  return self.volumebar;
}

void
set_background_pixmap (const gchar *filename, GtkWidget *widget,
                       gint *width_b, gint *height_b)
{
  GdkPixbuf *pbuf, *bg;
  GdkPixmap *pixmap;
  GdkBitmap *mask;
  gint height, width;

  if (!GTK_WIDGET_REALIZED (widget))
    gtk_widget_realize (widget);

  if (filename == NULL)
  {
    gdk_window_set_back_pixmap (widget->window, NULL, FALSE);
    return;
  }

  pbuf = gdk_pixbuf_new_from_file (filename, NULL);

  if (pbuf == NULL)
    return;

  width = gdk_pixbuf_get_width (pbuf);
  height = gdk_pixbuf_get_height (pbuf);
  *width_b = width;
  *height_b = height;


  gtk_widget_set_app_paintable (widget, TRUE);
  gdk_pixbuf_render_pixmap_and_mask (pbuf, &pixmap, &mask, 100);

  gdk_window_shape_combine_mask (widget->window, mask, 0, 0);
  gdk_window_set_back_pixmap (widget->window, pixmap, FALSE);
  gdk_pixbuf_unref (pbuf);
  gtk_widget_queue_draw (widget);
}

GtkWidget *
load_bgimage_to_pixmap (gchar *filename)
{
  GtkWidget *window;
  GdkPixbuf *pbuf;
  GdkPixmap *pixmap;
  GtkWidget *bgimage;

  window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
  gtk_widget_realize (window);
  pbuf = gdk_pixbuf_new_from_file (filename, NULL);
  if (pbuf == NULL)
    return NULL;
  gint width, height;

  width = gdk_pixbuf_get_width (pbuf);
  height = gdk_pixbuf_get_height (pbuf);
  pixmap = gdk_pixmap_new (window->window, width, height, -1);
  gdk_draw_pixbuf (pixmap, NULL, pbuf, 0, 0, 0, 0, -1, -1,
                   GDK_RGB_DITHER_NORMAL, 1, 1);

  self.pixbuf_bg = pbuf;
  self.pixmap = pixmap;
  self.gc = window->style->white_gc;
  bgimage = gtk_image_new_from_pixmap (pixmap, NULL);
  return bgimage;
}

GtkWidget *
create_volumebar_iner ()
{

  GtkWidget *image_bg, *image_sound_loud, *image_sound_hand, *image_roller;
  GtkWidget *volumebar_area = gtk_fixed_new ();
  GtkWidget *label_vol;

  gchar dir[100];

  strcpy (dir, IMAGE_DIR);
  strcat (dir, "/volume_bg.png");
  image_bg = load_bgimage_to_pixmap (dir);
  self.image_bg = image_bg;

  strcpy (dir, IMAGE_DIR);
  strcat (dir, "/icon_loudspker.png");
  image_sound_loud = gtk_image_new_from_file (dir);

  strcpy (dir, IMAGE_DIR);
  strcat (dir, "/icon_phone.png");
  image_sound_hand = gtk_image_new_from_file (dir);

  self.image_loudspeaker = image_sound_loud;
  self.image_handset = image_sound_hand;

  strcpy (dir, IMAGE_DIR);
  strcat (dir, "/roller.png");
  image_roller = gtk_image_new_from_file (dir);

  self.volumebar = volumebar_area;
  self.image_roller = image_roller;

  label_vol = gtk_label_new ("volume");
  PangoFontDescription *font_desc =
    pango_font_description_from_string ("Sans Bold 10");
  gtk_widget_modify_font (label_vol, font_desc);
  pango_font_description_free (font_desc);

  self.roller_pos_x = 160;
  self.roller_pos_y = 7;
  self.step = 10;
  self.left_bound = 110;
  self.right_bound = 210;
  gtk_fixed_put (GTK_FIXED (volumebar_area), image_bg, 0, 2);
  gtk_fixed_put (GTK_FIXED (volumebar_area), image_sound_loud, 2, 2);
  gtk_fixed_put (GTK_FIXED (volumebar_area), image_sound_hand, 2, 2);
  gtk_fixed_put (GTK_FIXED (volumebar_area), label_vol, 30, 5);
  gtk_fixed_put (GTK_FIXED (volumebar_area), image_roller,
                 self.roller_pos_x, self.roller_pos_y);
  gdk_draw_rectangle (self.pixmap, self.gc, TRUE, 114, 9,
                      self.roller_pos_x - self.left_bound, 5);
  gtk_widget_show_all (volumebar_area);
  gtk_widget_hide (self.image_loudspeaker);
  return volumebar_area;
}

gint
volume_changer (gboolean forward)
{
  if (forward)
    self.roller_pos_x += self.step;
  else
    self.roller_pos_x -= self.step;
  if (self.roller_pos_x >= self.right_bound)
  {
    self.roller_pos_x = self.right_bound;
  }
  else if (self.roller_pos_x <= self.left_bound)
  {
    self.roller_pos_x = self.left_bound;
  }
  gtk_fixed_move (GTK_FIXED (self.volumebar), self.image_roller,
                  self.roller_pos_x, self.roller_pos_y);
  gdk_draw_pixbuf (self.pixmap, NULL, self.pixbuf_bg, 114, 9, 114, 9,
                   self.right_bound - self.left_bound, 5,
                   GDK_RGB_DITHER_NORMAL, 1, 1);
  gdk_draw_rectangle (self.pixmap, self.gc, TRUE, 114, 9,
                      self.roller_pos_x - self.left_bound, 5);
  set_volume_to_mixer ((self.roller_pos_x -
                        self.left_bound) *100 / (self.right_bound -
                                                  self.left_bound));
  return TRUE;
}

gint
volume_set_to_min (gboolean ismute)
{
  if (ismute)
  {
    gtk_fixed_move (GTK_FIXED (self.volumebar), self.image_roller,
                    self.left_bound, self.roller_pos_y);
    gdk_draw_pixbuf (self.pixmap, NULL, self.pixbuf_bg, 114, 9, 114, 9,
                     self.right_bound - self.left_bound, 5,
                     GDK_RGB_DITHER_NORMAL, 1, 1);
  }

  else
  {
    gtk_fixed_move (GTK_FIXED (self.volumebar), self.image_roller,
                    self.roller_pos_x, self.roller_pos_y);
    gdk_draw_rectangle (self.pixmap, self.gc, TRUE, 114, 9,
                        self.roller_pos_x - self.left_bound, 5);
  }
  return TRUE;
}

void
volume_image_loudspeaker (gboolean isloudspker)
{
  if (isloudspker)
  {
    gtk_widget_show (self.image_loudspeaker);
    gtk_widget_hide (self.image_handset);
    create_msgdialog ("LoudSpeaker on!", MSG_NORMAL_TIMEOUT);
  }
  else
  {
    gtk_widget_hide (self.image_loudspeaker);
    gtk_widget_show (self.image_handset);
    create_msgdialog ("LoudSpeaker off!", MSG_NORMAL_TIMEOUT);
  }
}

void
set_volume_to_mixer (gint value)
{
  char *names[SOUND_MIXER_NRDEVICES] = SOUND_DEVICE_NAMES;
  int devmask = 0, recmask = 0, recsrc = 0;

  int ctrl, mode, arg, mixerfd, dev, left, right;
  gchar *channal = "vol";

  //"vol" "speaker"
  char name[30] = "/dev/mixer";

  if ((mixerfd = open (name, O_RDWR)) < 0)
  {
    perror (name);
    exit (1);
  }
  if (ioctl (mixerfd, SOUND_MIXER_READ_DEVMASK, &devmask) == -1)
  {
    perror ("SOUND_MIXER_READ_DEVMASK");
    exit (-1);
  }
  if (ioctl (mixerfd, SOUND_MIXER_READ_RECMASK, &recmask) == -1)
  {
    perror ("SOUND_MIXER_READ_RECMASK");
    exit (-1);
  }
  if (ioctl (mixerfd, SOUND_MIXER_READ_RECSRC, &recsrc) == -1)
  {
    perror ("SOUND_MIXER_READ_RECSRC");
    exit (-1);
  }
  for (ctrl = 0;
       ctrl < SOUND_MIXER_NRDEVICES && strcmp (names[ctrl], channal); ctrl++);



  left = right = value;

  if (left < 0)
    left = 0;
  if (right < 0)
    right = 0;
  if (left > 100)
    left = 100;
  if (right > 100)
    right = 100;


  arg = left | (right << 8);
  if (ioctl (mixerfd, MIXER_WRITE (ctrl), &arg) == -1)
  {
    perror ("MIXER_WRITE");
    exit (-1);
  }
  printf ("Setting the mixer %s to %d:%d.\n", names[ctrl], left, right);
  close (mixerfd);
}

/* 
vi:ts=2:nowrap:ai:expandtab
*/

